-- RELEASE ERROR REPORTS

function love.releaseerrhand(msg)
	msg = tostring(msg)
	
	print("An error has occured, the game has been stopped.")

    if not love.graphics or not love.event or not love.graphics.isCreated() then
        return
    end

    love.graphics.setCanvas()
    love.graphics.setPixelEffect()

    -- Load.
    if love.audio then love.audio.stop() end
    love.graphics.reset()
    love.graphics.setBackgroundColor(89, 157, 220)
    local font = love.graphics.newFont(14)
    love.graphics.setFont(font)

    love.graphics.setColor(255, 255, 255, 255)

    love.graphics.clear()

    local err = {}
	
	table.insert(err, "\n\nError\n")
    table.insert(err, msg)

    p = string.format("An error has occured that caused %s to stop.\n\nYou can notify %s about this%s.\nPlease include the text below in your email.", love._release.title or "this game", love._release.author or "the author", love._release.url and " at " .. love._release.url or "")
	p = p.."\n"..table.concat(err)
	
    local function draw()
        love.graphics.clear()
        love.graphics.printf(p, 70, 70, love.graphics.getWidth() - 70)
        love.graphics.present()
    end

    draw()

    local e, a, b, c
    while true do
        e, a, b, c = love.event.wait()

        if e == "quit" then
            return
        end
        if e == "keypressed" and a == "escape" then
            return
        end

        draw()

    end
end

-- BASE SECTION
pretitles = {
	{x=0,y=0,w=640,h=480,action={"skipintro"}},
}

menu = {
	{x=0,y=0,w=640,h=480,action={"null"}},
	{x=60,y=130,w=205,h=25,action={"game"}},
	{x=300,y=130,w=115,h=25,action={"restart"}},
	{x=60,y=160,w=160,h=25,action={"highscores"}},
	{x=60,y=210,w=65,h=25,action={"quit"}},
	{x=100,y=270,w=24,h=24,action={"quakesounds"}},
	{x=205,y=270,w=24,h=24,action={"digdugmode"}},
	{x=310,y=270,w=24,h=24,action={"hardmode"}},
	{x=390,y=270,w=24,h=24,action={"fullscreen"}},
	{x=470,y=270,w=24,h=24,action={"nightmode"}},
	{x=561,y=401,w=24,h=24,action={"resetsave"}},
}

game = {
	{x=0,y=0,w=640,h=480,action={"null"}},
}

tablehighscores = {
	{x=0,y=0,w=640,h=480,action={"null"}},
	{x=60,y=400,w=100,h=25,action={"return"}},
}

gameover = {
	{x=0,y=0,w=640,h=480,action={"null"}},
	{x=60,y=195,w=400,h=25,action={"highscore"}},
	{x=60,y=400,w=100,h=25,action={"return"}},
}

function arrowAdd(x,y)
	if spree < 140 then
		table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,y}})
		weapon = "bow"
	elseif spree >= 140 and spree < 190 then
		weapon = "bi-bow"
		if y == 0 then
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-65},vel={x,y}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,y}})
		elseif x == 0 then
			table.insert(arrows,{pos={pc.pos[1]+12,pc.pos[2]-40},vel={x,y}})
			table.insert(arrows,{pos={pc.pos[1]-12,pc.pos[2]-40},vel={x,y}})
		end
	elseif spree >= 230 and spree < 260 then
		weapon = "dualcaster"
		if x == 0 then
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={0.5,y}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={-0.5,y}})
		elseif y == 0 then
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,0.5}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,-0.5}})
		end
	elseif spree >= 300 then
		weapon = "tricaster"
		if x == 0 then
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={0.5,y}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,y}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={-0.5,y}})
		elseif y == 0 then
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,0.5}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,y}})
			table.insert(arrows,{pos={pc.pos[1],pc.pos[2]-40},vel={x,-0.5}})
		end
	end
end

function playerStop()
	pc.vel = {0,0}
end

function playerSetDirection(dir) 
	if(dir=="E") then pc.vel  = { 1, 0} player.direction=0 end
	if(dir=="S") then pc.vel  = { 0, 1} player.direction=1 end
	if(dir=="W") then pc.vel  = {-1, 0} player.direction=2 end
	if(dir=="N") then pc.vel  = { 0,-1} player.direction=3 end
	if pc.pos[1] < 10 then
		pc.pos[1] = 10
		pc.vel  = { 0, 0}
	elseif pc.pos[1] > 600 then
		pc.pos[1] = 600
		pc.vel  = { 0, 0}
	elseif pc.pos[2] > 460 then
		pc.pos[2] = 460
		pc.vel  = { 0, 0}
	elseif pc.pos[2] < 305 then
		pc.pos[2] = 305
		pc.vel  = { 0, 0}
	end
end

function objectUpdate(dt)
	if settings.digdugmode and pc.vel[1] == 0 and pc.vel[2] == 0 then
		return
	end
	for i,v in ipairs(objects) do
		-- stop animation
		if v.vel[1] == 0 and v.vel[2] == 0  then 
			v.phase = 0
		end
			
		-- move
		v.pos[1] = v.pos[1] + v.vel[1] * speed * dt
		v.pos[2] = v.pos[2] + v.vel[2] * speed * dt

		-- animate
		v.phase = (v.phase + dt*(25 - (worldsize/2))) %4
	end
	-- stop animation
	if pc.vel[1] == 0 and pc.vel[2] == 0  then 
		pc.phase = 0
	end

	-- move
	pc.pos[1] = pc.pos[1] + pc.vel[1] * speed * dt
	pc.pos[2] = pc.pos[2] + pc.vel[2] * speed * dt

	-- animate
	pc.phase = (pc.phase + dt*(25 - (worldsize/2))) %4
	
	for i,v in ipairs(earlyobjects) do
		-- stop animation
		if (v.vel[1] == 0 and v.vel[2] == 0 ) then 
			v.phase = 0
		end
			
			-- move
		v.pos[1] = v.pos[1] + v.vel[1] * (speed/2) * dt
		v.pos[2] = v.pos[2] + v.vel[2] * (speed/2) * dt
		
		-- animate
		v.phase = (v.phase + dt*(25 - (worldsize/2))) %4
	end
	for i,v in ipairs(earlyobjects) do
		if v.pos[1] > 10 and v.pos[1] < 610 and v.pos[2] > 305 then
			table.insert(objects,earlyobjects[i])
			table.remove(earlyobjects,i)
		end
	end
end

function arrowUpdate(dt)
	if settings.digdugmode and pc.vel[1] == 0 and pc.vel[2] == 0 then
		return
	end
	for i,v in ipairs(arrows) do
		v.pos[1] = v.pos[1] + v.vel[1] * (1*(speed/1)) * dt
		v.pos[2] = v.pos[2] + v.vel[2] * (1*(speed/1)) * dt
		
		if v.pos[1] < -100 or v.pos[2] > 700 or v.pos[2] < - 100 or v.pos[2] > 500 then
			table.remove(arrows,i)
		end
	end
end

function deerBrain()
	if settings.digdugmode and pc.vel[1] == 0 and pc.vel[2] == 0 then
		return
	end
	for i,v in ipairs(objects) do
		if v.image == deer then
			if tick > 1 then
				v.status = v.status + 1
			end
			if math.random(status,100) == 100 then
				local tempor = math.random(1,2)
				if tempor == 1 then
					local temp = math.random(-1,1)
					v.vel[1],v.vel[2] = temp,0
				elseif tempor == 2 then
					local temp = math.random(-1,1)
					v.vel[1],v.vel[2] = 0,temp
				end
				v.status = 0
				tempor = nil
			end
			if v.status == 99 then
				v.status = 0
			end
			if v.pos[1] < 10 then
				v.vel = {1,0}
			elseif v.pos[1] > 610 then
				v.vel = {-1,0}
			elseif v.pos[2] < 305 then
				v.vel = {0,1}
			elseif v.pos[2] > 470 then
				v.vel = {0,-1}
			end
		elseif v.image == bear then
			if tick > 1 then
				v.status = v.status + 1
			end
			if v.status > 2 then
				if math.mod(math.abs(status),math.random(0,10)) == 0 then
					local tempor = math.random(1,2)
					if tempor == 1 then
						if pc.pos[1] < v.pos[1] then
							v.vel[1],v.vel[2] = -1,0
						elseif pc.pos[1] > v.pos[1] then
							v.vel[1],v.vel[2] = 1,0
						end
					elseif tempor == 2 then
						if pc.pos[2] < v.pos[2] then
							v.vel[1],v.vel[2] = 0,-1
						elseif pc.pos[2] > v.pos[2] then
							v.vel[1],v.vel[2] = 0,1
						end
					end
					v.status = 0
					tempor = nil
				end
			end
			if v.pos[1] < 10 then
				v.vel = {1,0}
			elseif v.pos[1] > 610 then
				v.vel = {-1,0}
			elseif v.pos[2] < 305 then
				v.vel = {0,1}
			elseif v.pos[2] > 470 then
				v.vel = {0,-1}
			end
		elseif v.image == hog then
			if tick > 1 then
				v.status = v.status + 1
			end
			if v.status > 2 then
				if math.mod(math.abs(status),math.random(0,10)) == 0 then
					local tempor = math.random(1,2)
					if #pickupobjs == 0 then
						if tempor == 1 then
							local temp = math.random(-1,1)
							v.vel[1],v.vel[2] = temp,0
						elseif tempor == 2 then
							local temp = math.random(-1,1)
							v.vel[1],v.vel[2] = 0,temp
						end
					else
						local temp2 = nearest(v.pos[1],v.pos[2],pickupobjs)
						temp2 = {pickupobjs[temp2].pos[1],pickupobjs[temp2].pos[2]}
						if tempor == 1 then
							if temp2[1] < v.pos[1] then
								v.vel[1],v.vel[2] = -1,0
							elseif temp2[1] > v.pos[1] then
								v.vel[1],v.vel[2] = 1,0
							end
						elseif tempor == 2 then
							if temp2[2] < v.pos[2] then
								v.vel[1],v.vel[2] = 0,-1
							elseif temp2[2] > v.pos[2] then
								v.vel[1],v.vel[2] = 0,1
							end
						end					
					end
					v.status = 0
					tempor = nil
				end
			end
			if v.pos[1] < 10 then
				v.vel = {1,0}
			elseif v.pos[1] > 610 then
				v.vel = {-1,0}
			elseif v.pos[2] < 305 then
				v.vel = {0,1}
			elseif v.pos[2] > 470 then
				v.vel = {0,-1}
			end
		end
	end
end

function deerSpawn()
	-- DECIDING WHAT GETS SPAWNED
	local spawnnumber = level
	local spawnitems = {}
	if spawnnumber < 10 then
		for i = 1,spawnnumber do
			table.insert(spawnitems,1)
		end
	elseif spawnnumber > 9 then
		for i = 1,spawnnumber do
			local temp = math.random(1,100)
			if temp <= 20 then
				table.insert(spawnitems,2)
			elseif spawnnumber > 19 and temp>20 and temp <= 30 then
				table.insert(spawnitems,3)
			else
				table.insert(spawnitems,1)
			end
			temp = nil
		end
	end
	local temp = 0
	for i,v in ipairs(spawnitems) do
		temp = temp + v
	end
	if temp > spawnnumber then
		repeat
			temp = temp - spawnitems[#spawnitems]
			table.remove(spawnitems)
		until temp <= spawnnumber
	end
	
	-- ACTUALLY SPAWNING IT
	for i = 1,#spawnitems do
		if level < 3 then
			table.insert(earlyobjects,{image = deer, pos = {320,210}, vel = {0,1}, phase = 0,status = 0})
		elseif level >= 3 then
			local temp2 = math.random(1,3)
			if temp2 == 1 then
				if spawnitems[i] == 1 then
					table.insert(earlyobjects,{image = deer, pos = {320,210}, vel = {0,1}, phase = 0,status = 0})
				elseif spawnitems[i] == 2 then
					table.insert(earlyobjects,{image = bear, pos = {320,210}, vel = {0,1}, phase = 0,status = 0})
				elseif spawnitems[i] == 3 then
					table.insert(earlyobjects,{image = hog, pos = {320,210}, vel = {0,1}, phase = 0,status = 0})
				end
			elseif temp2 == 2 then
				if spawnitems[i] == 1 then
					table.insert(earlyobjects,{image = deer, pos = {-100,380}, vel = {1,0}, phase = 0,status = 0})
				elseif spawnitems[i] == 2 then
					table.insert(earlyobjects,{image = bear, pos = {-100,380}, vel = {1,0}, phase = 0,status = 0})
				elseif spawnitems[i] == 3 then
					table.insert(earlyobjects,{image = hog, pos = {-100,380}, vel = {1,0}, phase = 0,status = 0})
				end
			elseif temp2 == 3 then
				if spawnitems[i] == 1 then
					table.insert(earlyobjects,{image = deer, pos = {700,380}, vel = {-1,0}, phase = 0,status = 0})
				elseif spawnitems[i] == 2 then
					table.insert(earlyobjects,{image = bear, pos = {700,380}, vel = {-1,0}, phase = 0,status = 0})
				elseif spawnitems[i] == 3 then
					table.insert(earlyobjects,{image = hog, pos = {700,380}, vel = {-1,0}, phase = 0,status = 0})
				end
			end
		end
	end
	spawnnumber = nil
	spawnitems = nil
end

function scoresort (a,b)
	return a.score>b.score
end

function collisionDetect(dt)
	if settings.digdugmode and pc.vel[1] == 0 and pc.vel[2] == 0 then
		return
	end
	for j,k in ipairs(objects) do
		for i,v in ipairs(arrows) do
			if checkCollision(v.pos[1]-15, v.pos[2]-15, 30, 30, k.pos[1]-40, k.pos[2]-40, 90, 40) then
				local temp = math.random(1,1000)
				if temp ~= nil and temp ~= null and objects[j] ~= null then
					if temp <= 20 then
						table.insert(pickupobjs,{pos={objects[j].pos[1],objects[j].pos[2]},quad=1,ticker=0})
					elseif temp > 20 and temp <= 30 then
						table.insert(pickupobjs,{pos={objects[j].pos[1],objects[j].pos[2]},quad=2,ticker=0})
					elseif temp > 30 and temp <= 40 then
						table.insert(pickupobjs,{pos={objects[j].pos[1],objects[j].pos[2]},quad=3,ticker=0})
					elseif temp > 40 and temp <= 45 then
						table.insert(pickupobjs,{pos={objects[j].pos[1],objects[j].pos[2]},quad=4,ticker=0})
					end
				end
				love.audio.play(shot)
				if k.image == deer then
					deads.deer = deads.deer + 1
				elseif k.image == bear then
					deads.bears = deads.bears + 1
				elseif k.image == hog then
					deads.hogs = deads.hogs + 1
				end
				spree = spree + 1
				if spree > highest then
					highest = spree
				end
				bannerAdd()
				temp = nil
				local temp = false
				local temp2 = 0
				for i = #highscores,1,-1 do
					if ((highest+deads.deer+deads.bears+deads.hogs)/10)*level > highscores[i].score then
						temp=true
						temp2=i
					end
				end
				if temp then
					if temp2 > scoreposition then
						table.insert(bannertexts,{"YOU ARE "..temp2.." !"})
					end
					scoreposition = temp2
				end
				temp = nil
				temp2 = nil
				table.remove(arrows,i)
				table.remove(objects,j)
			end
		end
		if checkCollision(pc.pos[1]-20, pc.pos[2]-0, 4, 4, k.pos[1]-40, k.pos[2]-40, 90, 20) and not painon then
			pc.pos[1] = pc.pos[1] +cs
			health = health - damage
			painon = true
			tick = 0
			if spree > 21 then
				spree = spree - 10
			else
				spree = 2
			end
			local temp = math.random(1,10)
			if temp == 1 then
				table.insert(currentemotes,6)
			end
			temp = nil
		elseif checkCollision(pc.pos[1]+20, pc.pos[2]-0, 4, 4, k.pos[1]-40, k.pos[2]-40, 90, 20) and not painon then
			pc.pos[1] = pc.pos[1] -cs
			health = health - damage
			painon = true
			tick = 0
			if spree > 21 then
				spree = spree - 10
			else
				spree = 2
			end
			local temp = math.random(1,10)
			if temp == 1 then
				table.insert(currentemotes,6)
			end
			temp = nil
		elseif checkCollision(pc.pos[1]-0, pc.pos[2]+10, 4, 4, k.pos[1]-40, k.pos[2]-40, 90, 20) and not painon then
			pc.pos[2] = pc.pos[2] -cs
			health = health - damage
			painon = true
			tick = 0
			if spree > 21 then
				spree = spree - 10
			else
				spree = 2
			end
			local temp = math.random(1,10)
			if temp == 1 then
				table.insert(currentemotes,6)
			end
			temp = nil
		elseif checkCollision(pc.pos[1]-0, pc.pos[2]-20, 4, 4, k.pos[1]-40, k.pos[2]-40, 90, 20) and not painon then
			pc.pos[2] = pc.pos[2] +cs
			health = health - damage
			painon = true
			tick = 0
			if spree > 11 then
				spree = spree - 10
			else
				spree = 2
			end
			local temp = math.random(1,10)
			if temp == 1 then
				table.insert(currentemotes,6)
			end
			temp = nil
		end
		if k.image == hog and #pickupobjs > 0 then
			for i,v in ipairs(pickupobjs) do
				if checkCollision(k.pos[1]-40,k.pos[2]-40,90,20,v.pos[1]-15,v.pos[2]-15,30,30) then
					table.remove(pickupobjs,i)
				end
			end
		end
	end
	for i,v in ipairs(pickupobjs) do
		v.ticker = v.ticker + 1*dt
		if v.ticker > 4 then
			table.remove(pickupobjs,i)
		end
		if checkCollision(pc.pos[1]-40, pc.pos[2]-20, 80, 40, v.pos[1]-15, v.pos[2]-15, 30, 30) then
			if v.quad == 1 then
				if health > 80 then
					health = 100
				elseif health <= 80 then
					health = health + 20
				end
				table.remove(pickupobjs,i)
				table.insert(currentemotes,5)
			elseif v.quad == 2 then
				spree = spree + 10
				table.remove(pickupobjs,i)
				table.insert(currentemotes,4)
			elseif v.quad == 3 then
				speed = speed + (10-(level/5))
				spree = spree + 30
				health = health + 10
				table.remove(pickupobjs,i)
				table.insert(currentemotes,4)
			elseif v.quad == 4 then
				objects = {}
				table.remove(pickupobjs,i)
				table.insert(currentemotes,7)
				if settings.nightmode and not flare[1] then
					flare = {true,0.1,0.1,0,0,pc.pos[1],pc.pos[2]}
				end
			end
		end
	end
	if flare[1] then
		flare[2] = flare[2] + (flare[3]*dt)
		if flare[2] > 2 then
			flare[3] = -0.1
		elseif flare[2] <= 0 then
			flare = {false,0,0.1,0,0,pc.pos[1],pc.pos[2]}
		end
		if flare[3] == 0.1 then
			flare[5] = flare[5]-(400*dt)
			flare[4] = flare[4]-(10*dt)
		end
	end
end

function bannerAdd()
	if spree == 1 then
		table.insert(bannertexts,{"FIRST BLOOD!",1})
	elseif spree == 5 then
		table.insert(bannertexts,{"MULTIKILL!",2})
	elseif spree == 15 then
		table.insert(bannertexts,{"KILLINGSPREE!",3})
		table.insert(currentemotes,1)
	elseif spree == 30 then
		table.insert(bannertexts,{"DOMINATING!",4})
	elseif spree == 50 and settings.nightmode then
		table.insert(bannertexts,{"PICKUP INSIGHT!"})
	elseif spree == 60 then
		table.insert(bannertexts,{"RAMPAGE!",5})
	elseif spree == 70 then
		table.insert(bannertexts,{"WICKEDSICK!",6})
	elseif spree == 90 and settings.nightmode then
		table.insert(bannertexts,{"NIGHT SIGHT!"})
	elseif spree == 110 then
		table.insert(bannertexts,{"BI-BOW UNLOCKED!"})
		table.insert(currentemotes,7)
	elseif spree == 190 then
		table.insert(bannertexts,{"DUALCASTER UNLOCKED!"})
		table.insert(currentemotes,7)
	elseif spree == 260 then
		table.insert(bannertexts,{"TRICASTER UNLOCKED!"})
		table.insert(currentemotes,4)
	elseif spree == 100 then
		table.insert(bannertexts,{"LUDICROUSKILL!",7})
	elseif spree == 150 then
		table.insert(bannertexts,{"MONSTERKILL!",8})
		table.insert(currentemotes,2)
	elseif spree == 170 then
		table.insert(bannertexts,{"ULTRAKILL!",9})
	elseif spree == 220 then
		table.insert(bannertexts,{"UNSTOPPABLE!",10})
		table.insert(currentemotes,2)
	elseif spree == 300 then
		table.insert(bannertexts,{"GODLIKE!",11})
	end
end

function emotePush()
	if #currentemotes > 0 then
		emotetimer = 0
		emote = currentemotes[1]
		table.remove(currentemotes,1)
	end
	if emotetimer > 1 then
		emote = "null"
	end
end

function emoteDisplay()
	if emote ~= "null" then
		love.graphics.drawq(emotesimage,emotes[emote],pc.pos[1],pc.pos[2],0,worldsize/2,worldsize/2,9,32)
	end
end

function bannerPush()
	if #bannertexts > 0 then
		bannertimer = 0
		banner = bannertexts[1][1]
		if bannertexts[1][2] ~= null then
			if settings.quakesounds then
				if bannertexts[1][2] ~= 1 then
					love.audio.stop(quakesounds[bannertexts[1][2]-1])
				end
				love.audio.play(quakesounds[bannertexts[1][2]])
			end
		end
		table.remove(bannertexts,1)
	end
	if bannertimer > 1 then
		banner = " "
	end
end

function colourreverse(data)
	local output = {}
	for i,v in ipairs(data) do
		table.insert(output,math.abs(v-255))
	end
	return output
end

require "ow_control"
require "tl_libs/tl_libs"

function loadAssets()
	math.randomseed(os.time())	
	require "ow_rope"
	rope.read("illusion.lua")
	
	TLbind.keys = {home = "return", f1 = "debug", escape = "escape", w = "sup", a = "sleft", s = "sdown", d = "sright", up = "up", left = "left", down = "down", right = "right"}
	TLbind.joyHats = { {{"left", "right", "up", "down"}} }
	TLbind.joyBtns = { {"sdown", "sright", "sleft", "sup",[8]="escape"} }
	
	--timers = rope.readandreturn("ow_timers.lua")
	quakesounds = {
		--[[firstblood ]] love.audio.newSource("sounds/firstblood.wav","stream"),
		--[[multikill ]] love.audio.newSource("sounds/multikill.wav","stream"),
		--[[killingspree ]] love.audio.newSource("sounds/killingspree.wav","stream"),
		--[[dominating ]] love.audio.newSource("sounds/dominating.wav","stream"),
		--[[rampage ]] love.audio.newSource("sounds/rampage.wav","stream"),
		--[[wickedsick ]] love.audio.newSource("sounds/wickedsick.wav","stream"),
		--[[ludicrouskill ]] love.audio.newSource("sounds/ludicrouskill.wav","stream"),
		--[[monsterkill ]] love.audio.newSource("sounds/monsterkill.wav","stream"),
		--[[ultrakill ]] love.audio.newSource("sounds/ultrakill.wav","stream"),
		--[[unstoppable ]] love.audio.newSource("sounds/unstoppable.wav","stream"),
		--[[godlike ]] love.audio.newSource("sounds/godlike.wav","stream"),
	}
	emotes = {
		love.graphics.newQuad(0,0,15,14,105,14),	--happy
		love.graphics.newQuad(15,0,15,14,105,14),	--smiley
		love.graphics.newQuad(30,0,15,14,105,14),	--question
		love.graphics.newQuad(45,0,15,14,105,14),	--suprise
		love.graphics.newQuad(60,0,15,14,105,14),	--love
		love.graphics.newQuad(75,0,15,14,105,14),	--pain
		love.graphics.newQuad(90,0,15,14,105,14)	--cat
	}
	arrowqs = {
		love.graphics.newQuad(0,0,6,6,6,26),
		love.graphics.newQuad(1,6,5,6,6,26),
		love.graphics.newQuad(0,13,6,6,6,26),
		love.graphics.newQuad(1,19,5,6,6,26),
	}
	pickupqs = {
		love.graphics.newQuad(0,0,6,6,6,26), -- health
		love.graphics.newQuad(0,6,6,6,6,26), -- +10
		love.graphics.newQuad(0,13,6,6,6,26),-- speed
		love.graphics.newQuad(0,19,6,6,6,26),-- bomb
	}
	
	play = love.audio.newSource("sounds/play.wav","stream")
	shot = love.audio.newSource("sounds/shot.ogg","static")
	shot2 = love.audio.newSource("sounds/shot2.ogg","static")
	
	debugfont = love.graphics.newFont("Lekton-Bold.ttf",11)
	font14 = love.graphics.newFont("Lekton-Bold.ttf",14)
	font48 = love.graphics.newFont("Lekton-Bold.ttf",48)
	font24 = love.graphics.newFont("Lekton-Bold.ttf",24)
	
	backgroundscene = love.graphics.newImage("backgroundscene.png")
	emotesimage = love.graphics.newImage("emotes.png")
	emotesimage:setFilter( "nearest", "nearest" )
	pickupsimage = love.graphics.newImage("pickups.png")
	pickupsimage:setFilter( "nearest", "nearest" )
	char = love.graphics.newImage("char.png")
	char:setFilter( "nearest", "nearest" )
	shad = love.graphics.newImage("shadow.png")
	deer = love.graphics.newImage("deer.png")
	deer:setFilter( "nearest", "nearest" )
	bear = love.graphics.newImage("bear.png")
	bear:setFilter( "nearest", "nearest" )
	hog = love.graphics.newImage("piggyog.png")
	hog:setFilter( "nearest", "nearest" )
	arrow = love.graphics.newImage("arrow.png")
	arrow:setFilter("nearest","nearest")
	sun = love.graphics.newImage("sun.png")
	logo = love.graphics.newImage("logo.png")
	menuback = love.graphics.newImage("menuback.png")
	
	flare = {false,0,0.1,0,0,0,0}
end

-- load up love.load removed for OW_HUB compatibility
function love.load()
	loadAssets()
	
	-- setup vars
	
	sliderinfo = {{0,0,100},{0,255,100},}
	colours = {{0,255,100}}
	--colours[2] = colourreverse(colours[1])
	colours[2] = {0,0,0}
	health = 100
	level = 1
	earlyobjects = {}
	pc = {image = char, pos = {310,380}, vel = {0,0}, phase = 0}
	objects = {}
	arrows = {}
	debug = false
	typedinput = false
	typed = {}
	tick = 0
	status = 0
	speed = 240
	spawned = false
	player = {direction = 0}
	worldsize = 10
	spree = 0
	highest = 0
	datum = 2
	cs = 30
	deads = {deer=0,bears=0,hogs=0}
	damage = 5
	painon = false
	pickupobjs = {}
	bannertexts = {}
	bannertimer = 0
	banner = " "
	weapon = "bow"
	reload = 101
	firespeed = 100
	emotetimer = 0
	currentemotes = {}
	emote = "null"
	scoreposition = 0
	highscorename = " "
	fullscreen = false
	
	rope.load("obscureworlds/deerhunter1066","settings.lua")
	if settings.nightmode then
		canvas = love.graphics.newCanvas(840,680)
	end
	
	area = "pretitles"
	currenttable = pretitles
	
	-- setup res
	
	local temp = false
	
	if settings.fullscreen then
		temp = true
	end
	
	TLfres.setScreen({w=640,h=480,full=temp,vsync=false,aa=0}, 640, false, false)
	
	temp = nil
end
	

-- UPDATE SECTION

function love.update(dt)
	TLbind:update()
	--timers.update()
	slider(dt)
	
	mX,mY = love.mouse.getX(),love.mouse.getY()

	if area ~= "game" then if not painon then status = 0 end end
	if area == "pretitles" then
		if sliderinfo[2][1] == sliderinfo[2][2] then
			sliderinfo[2][1] = sliderinfo[2][1] - 1
			sliderinfo[2][2] = 0
			sliderinfo[2][3] = sliderinfo[2][3] * 2
			area = "menu"
			currenttable = menu
			sliderinfo[1][2] = 255
		end
	end
	
	if settings.digdugmode then
		if pc.vel[1] ~= 0 or pc.vel[2] ~= 0 then
			reload = reload + (firespeed+(spree*2))*dt
		end
	else
		reload = reload + ((firespeed+(spree*2))+level)*dt
	end
	
	playerStop()
	if status >= datum then
		if control["up"] then    playerSetDirection("N") end
		if control["down"] then  playerSetDirection("S") end
		if control["left"] then  playerSetDirection("W") end
		if control["right"] then playerSetDirection("E") end
	end
	objectUpdate(dt)
	deerBrain()
	arrowUpdate(dt)
	if area == "game" then
		collisionDetect(dt)
	end
	bannertimer = bannertimer + 1*dt
	emotetimer = emotetimer + 1*dt
	bannerPush()
	emotePush()
	if status == datum and spawned == false then
		if settings.quakesounds and level == 1 then
			table.insert(bannertexts,{"PLAY!"})
			love.audio.play(play)
		end
		spawned = true
		deerSpawn()
	end
	
	if health <= 0 then
		area = "gameover"
		currenttable = gameover
		health = 100
	end
	
	if #objects == 0 and #earlyobjects == 0 and status > datum and spawned == true then
		level = level + 1
		status = 0
		colours = {{colours[1][1],colours[1][2]+math.random(-5,0),colours[1][3]+5}}
		--colours[2] = colourreverse(colours[1])
		colours[2] = {0,0,0}
		spawned = false
		bannertexts = {}
		table.insert(bannertexts,{"LEVEL "..level.."!"})
		speed = speed + (5-(level/100))
	end
	
	if tick <= 2 then
		tick = tick + (2*dt)
	elseif tick > 2 then
		tick = 0
		status = status + 1
	end
	if tick > 1 then
		painon = false 
	end
	table.sort(objects,ysort)
	
	keydown()
	
end

-- DRAW SECTION

function areaDraw()
	if area == "pretitles" then
		love.graphics.setColor(255,255,255,sliderinfo[2][1])
		love.graphics.draw(logo,(love.graphics.getWidth()*0.5)-(logo:getWidth()*0.5),(love.graphics.getHeight()*0.5)-(logo:getHeight()*0.5))
	elseif area == "menu" then
		love.graphics.setColor(255,255,255,sliderinfo[2][1])
		love.graphics.draw(logo,(love.graphics.getWidth()*0.5)-(logo:getWidth()*0.5),(love.graphics.getHeight()*0.5)-(logo:getHeight()*0.5))
		love.graphics.setColor(255,255,255,sliderinfo[1][1])
		love.graphics.draw(menuback,40,50)
		--love.graphics.rectangle("fill",40,50,560,380)
		love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3],sliderinfo[1][1])
		love.graphics.setFont(font48)
		love.graphics.setColor(255,0,155,sliderinfo[1][1])
		if settings.nightmode then
			love.graphics.print("DeerPoacher 1066",(love.graphics.getWidth()*0.5)-(font48:getWidth("DeerPoacher 1066")*0.5),70)
		else
			love.graphics.print("DeerHunter 1066",(love.graphics.getWidth()*0.5)-(font48:getWidth("DeerHunter 1066")*0.5),70)
		end
		love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3],sliderinfo[1][1])
		love.graphics.setFont(font24)
		if deads.deer == 0 and not spawned then
			love.graphics.print("New Game",60,130)
		else
			love.graphics.print("Continue Game",60,130)
			love.graphics.print("Restart",300,130)
		end
		love.graphics.print("Highscores",60,160)
		love.graphics.print("Quit!",60,210)
		love.graphics.setFont(font14)
		if deads.deer == 0 and not spawned then
			love.graphics.print("Quake sounds. Stopstart mode. Hard mode. Fullscreen. Nightmode.",55,250)
			if settings.quakesounds then
				love.graphics.rectangle("fill",100,270,24,24)
			else
				love.graphics.rectangle("line",100,270,24,24)
			end
			if settings.digdugmode then
				love.graphics.rectangle("fill",205,270,24,24)
			else
				love.graphics.rectangle("line",205,270,24,24)
			end
			if settings.hardmode then
				love.graphics.rectangle("fill",310,270,24,24)
			else
				love.graphics.rectangle("line",310,270,24,24)
			end
			if settings.fullscreen then
				love.graphics.rectangle("fill",390,270,24,24)
			else
				love.graphics.rectangle("line",390,270,24,24)
			end
			if settings.nightmode then
				love.graphics.rectangle("fill",470,270,24,24)
			else
				love.graphics.rectangle("line",470,270,24,24)
			end
		end
		love.graphics.print("Reset save:",480,410)
		love.graphics.rectangle("line",561,401,24,24)
		love.graphics.printf("Direction keys for movement and wasd for arrows. Trouble seeing? tap enter to reset colourscheme.",60,380,520,"left")
		love.graphics.print("Prototype developed by ObscureWorlds for the LOVE engine.",60,410)
	elseif area == "highscores" then
		love.graphics.setColor(255,255,255,sliderinfo[1][1])
		love.graphics.draw(menuback,40,50)
		--love.graphics.rectangle("fill",40,50,560,380)
		love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3],sliderinfo[1][1])
		love.graphics.setFont(font24)
		love.graphics.print("DEERHUNTER1066 HIGHSCORES",60,70)
		love.graphics.print("RETURN",60,400)
		love.graphics.setFont(font14)
		for i,v in ipairs(highscores) do
			if i ~= 10 then
				love.graphics.print(i..": player: "..v.name..", score:"..v.score..", spree:"..v.highestspree..", mode: "..v.mode,60,100+(15*(i-1)))
			else
				love.graphics.print(i..": player: "..v.name..", score:"..v.score..", spree:"..v.highestspree..", mode: "..v.mode,51,100+(15*(i-1)))
			end
		end
	elseif area == "game" then
		love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3])
		for i,v in ipairs(pickupobjs) do
			love.graphics.drawq(pickupsimage,pickupqs[v.quad],v.pos[1],v.pos[2],0,worldsize,worldsize,3,6)
		end
		love.graphics.setColor(255,65,0)
		for i,v in ipairs(arrows) do
			if v.vel[1] == 1 then
				love.graphics.drawq(arrow,arrowqs[1],v.pos[1],v.pos[2],0,5,5,3,3)
			elseif v.vel[2] == 1 then
				love.graphics.drawq(arrow,arrowqs[2],v.pos[1],v.pos[2],0,5,5,3,3)
			elseif v.vel[1] == -1 then
				love.graphics.drawq(arrow,arrowqs[3],v.pos[1],v.pos[2],0,5,5,3,3)
			elseif v.vel[2] == -1 then
				love.graphics.drawq(arrow,arrowqs[4],v.pos[1],v.pos[2],0,5,5,3,3)
			end
		end
		if status < datum and tick > 1 then
			love.graphics.setColor(255,65,0)
			love.graphics.drawq(pc.image,love.graphics.newQuad(math.floor(pc.phase)*11,player.direction*12 , 11,12,44,48),pc.pos[1],pc.pos[2],0,worldsize,worldsize,10/2,11)
			love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3])
		end
		love.graphics.setColor(255,255,255)
		if tick > 1 then
			for i,v in ipairs(earlyobjects) do
				local dir = 0
				if v.vel[1] == 1 then
					dir = 0
				elseif v.vel[1] == -1 then
					dir = 2
				elseif v.vel[2] == 1 then
					dir = 1
				elseif v.vel[2] == -1 then
					dir = 3
				end
				if v.image == deer then
					love.graphics.drawq(v.image,love.graphics.newQuad(math.floor(v.phase)*11,dir*12 , 11,12,44,48),v.pos[1],v.pos[2],0,worldsize,worldsize,10/2,11)
				elseif v.image == bear then
					love.graphics.drawq(v.image,love.graphics.newQuad(math.floor(v.phase)*22,dir*24 , 22,24,88,96),v.pos[1],v.pos[2],0,worldsize/2,worldsize/2,20/2,22)
				elseif v.image == hog then
					love.graphics.drawq(v.image,love.graphics.newQuad(math.floor(v.phase)*44,dir*48 , 44,48,176,192),v.pos[1],v.pos[2],0,worldsize/3,worldsize/3,40/2,44)
				end
			end
		end
		if status >= datum then
			for i = #objects,1,-1 do
				local dir = 0
				if objects[i].vel[1] == 1 then
					dir = 0
				elseif objects[i].vel[1] == -1 then
					dir = 2
				elseif objects[i].vel[2] == 1 then
					dir = 1
				elseif objects[i].vel[2] == -1 then
					dir = 3
				end
				if objects[i].image == deer then
					love.graphics.drawq(objects[i].image,love.graphics.newQuad(math.floor(objects[i].phase)*11,dir*12 , 11,12,44,48),objects[i].pos[1],objects[i].pos[2],0,worldsize,worldsize,10/2,11)
				elseif objects[i].image == bear then
					love.graphics.drawq(objects[i].image,love.graphics.newQuad(math.floor(objects[i].phase)*22,dir*24 , 22,24,88,96),objects[i].pos[1],objects[i].pos[2],0,worldsize/2,worldsize/2,20/2,22)
				elseif objects[i].image == hog then
					love.graphics.drawq(objects[i].image,love.graphics.newQuad(math.floor(objects[i].phase)*44,dir*48 , 44,48,176,192),objects[i].pos[1],objects[i].pos[2],0,worldsize/3,worldsize/3,40/2,44)
				end
			end
			love.graphics.setColor(255,65,0)
			if not painon then
				love.graphics.drawq(pc.image,love.graphics.newQuad(math.floor(pc.phase)*11,player.direction*12 , 11,12,44,48),pc.pos[1],pc.pos[2],0,worldsize,worldsize,10/2,11)
			elseif painon then
				love.graphics.setColor(255,0,0)
				love.graphics.drawq(pc.image,love.graphics.newQuad(math.floor(pc.phase)*11,player.direction*12 , 11,12,44,48),pc.pos[1],pc.pos[2],0,worldsize,worldsize,10/2,11)
			end
			love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3])
		end
		
		
		
		if settings.nightmode then
		
			canvas:clear()
			love.graphics.setCanvas(canvas)
			love.graphics.setColor(255,255,255,250)
			love.graphics.rectangle("fill",0,0,800,600)
			love.graphics.setColor(0,0,0,255)
			if spree >= 90 then
				love.graphics.draw(shad,pc.pos[1]+20,pc.pos[2]+20,0,1.2+flare[2],1.25+flare[2],220,140)
			elseif spree < 90 then
				love.graphics.draw(shad,pc.pos[1]+20,pc.pos[2]+20,0,1+flare[2],1+flare[2],220,140)
			end
			if status < datum and tick > 1 then
				love.graphics.drawq(pc.image,love.graphics.newQuad(math.floor(pc.phase)*11,player.direction*12 , 11,12,44,48),pc.pos[1]+20,pc.pos[2]+20,0,worldsize,worldsize,10/2,11)
			elseif status >= datum and not painon then
				love.graphics.drawq(pc.image,love.graphics.newQuad(math.floor(pc.phase)*11,player.direction*12 , 11,12,44,48),pc.pos[1]+20,pc.pos[2]+20,0,worldsize,worldsize,10/2,11)
			elseif status >= datum and painon then
				love.graphics.drawq(pc.image,love.graphics.newQuad(math.floor(pc.phase)*11,player.direction*12 , 11,12,44,48),pc.pos[1]+20,pc.pos[2]+20,0,worldsize,worldsize,10/2,11)
			end
			if spree >= 50 then
				for i,v in ipairs(pickupobjs) do
					love.graphics.draw(shad,v.pos[1]+20,v.pos[2]-5,0,0.3,0.3,220,140)
				end
			end
			love.graphics.setColor(255,255,255,255)
			love.graphics.setCanvas()
			
			if flare[1] then
				love.graphics.setColor(255,100,100,100)
				love.graphics.rectangle("fill",-20,-20,840,640)
				love.graphics.setColor(255,255,255,255)
			end
			
			love.graphics.setBlendMode("subtractive")
			love.graphics.draw(canvas,-20,-20)
			love.graphics.setBlendMode("alpha")
			
			if flare[1] then
				love.graphics.setColor(255,200,200,180)
				love.graphics.draw(sun,flare[6]+flare[4],flare[7]+flare[5],flare[5]/2,flare[2]/2,flare[2]/2,227,248)
				love.graphics.draw(sun,flare[6]+flare[4],flare[7]+flare[5],-flare[5]/2,flare[2]/2,flare[2]/2,227,248)
				love.graphics.setColor(255,255,255,255)
			end
		end
		
		
		love.graphics.setColor(255,255,255)
		love.graphics.setFont(font48)
		love.graphics.print(banner,640/2-(font48:getWidth(banner)/2),50)
		love.graphics.setFont(font14)
		emoteDisplay()
	
		love.graphics.print("HEALTH:",0,465)
		if settings.digdugmode then
			love.graphics.print("STOPSTARTMODE: ACTIVE.",275,465)
		end
		love.graphics.print("SPREE: "..spree..".",470,465)
		love.graphics.print("LEVEL: "..level..".",560,465)
		if settings.hardmode then
			if reload > 100 then
				love.graphics.print("LOADED",5,25)
				love.graphics.rectangle("line",5,5,100,20)
				love.graphics.rectangle("fill",5,5,100,20)
			else
				love.graphics.print("RELOADING",5,25)
				love.graphics.rectangle("line",5,5,100,20)
				love.graphics.rectangle("fill",5,5,reload,20)
			end
		end
		love.graphics.print("POSITION: "..scoreposition,110,5)
		love.graphics.print("SCORE: "..((highest+deads.deer+deads.bears+deads.hogs)/10)*level,110,20)

		love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3])
		love.graphics.setColor(255,65,0)
		for i = 1,health do
			love.graphics.print("#",60+(2*(i-1)),465)
		end
		love.graphics.print(weapon,5,5)
		love.graphics.setColor(colours[1][1],colours[1][2],colours[1][3])
	elseif area == "gameover" then
		love.graphics.setColor(255,255,255)
		love.graphics.draw(menuback,40,50)
		--love.graphics.rectangle("fill",40,50,560,380)
		love.graphics.setColor(colours[2][1],colours[2][2],colours[2][3])
		love.graphics.setFont(font24)
		love.graphics.print("GAMEOVER ++ GAMEOVER",60,70)
		love.graphics.setFont(font14)
		love.graphics.print("You got to Level "..level..".",60,95)
		love.graphics.print("You killed "..deads.deer.." deer, "..deads.bears.." bears and "..deads.hogs.." hogs you monster!",60,120)
		love.graphics.print("Your score is "..((highest+deads.deer+deads.bears+deads.hogs)/10)*level..".",60,145)
		love.graphics.setFont(font24)
		if scoreposition > 0 then
			love.graphics.print("YOU ARE NUMBER "..scoreposition..".",60,170)
			if highscorename == " " and not typedinput then
				love.graphics.setFont(font14)
				love.graphics.print("click here and type your name to set a highscore.",60,195)
				love.graphics.setFont(font24)
			else
				love.graphics.print(highscorename,60,195)
				if typedinput then
					love.graphics.setFont(font14)
					love.graphics.print("click anywhere to finish typing.",60,220)
					love.graphics.setFont(font24)
				end
			end
		end
		love.graphics.print("RETURN",60,400)
	end
end

function love.draw()
	TLfres.transform()
	
	love.graphics.push()
	if painon then
		love.graphics.translate(math.random(-5,5),math.random(-5,5))
	end
	love.graphics.setFont(font14)
	love.graphics.setColor(colours[1][1],colours[1][2],colours[1][3],255)
	love.graphics.rectangle("fill",0,0,800,600)
	love.graphics.draw(backgroundscene,0,0)
	if settings.nightmode and area ~= "game" then
		love.graphics.setColor(0,0,0,250)
		love.graphics.rectangle("fill",0,0,800,600)
		love.graphics.setColor(255,255,255,255)
	end
	areaDraw()
	
	love.graphics.pop()
	if debug then
		if currenttable ~= null then
			if #currenttable > 1 then
				love.graphics.setColor(200,200,200,100)
				for i = 2,#currenttable do
					love.graphics.rectangle("fill",currenttable[i].x,currenttable[i].y,currenttable[i].w,currenttable[i].h)
				end
				love.graphics.setColor(255,255,255,255)
			end
		end
	
		love.graphics.setFont(debugfont)
		love.graphics.setColor(0,0,0,100)
		love.graphics.rectangle("fill",mX+5,mY+5,150,25)
		love.graphics.setColor(255,255,255,255)
		love.graphics.print("Mpos="..love.mouse.getX().." : "..love.mouse.getY(),mX+10,mY+5)
		love.graphics.print("fps="..love.timer.getFPS(),mX+10,mY+17)
		love.graphics.setFont(font14)
	end
	
	TLfres.letterbox(4,3)
end

-- INPUT SECTION

-- click area detection functions for menu code
function nearestMenu(x,y)
	current = nearestestui(x,y,currenttable)
	return current
end

function nearestestui(x,y,table)
	for i,v in ipairs(table) do
		if checkCollision(x,y,1,1,v.x,v.y,v.w,v.h) == true then
			id = i
		end
	end
	return id
end

function scoresort (a,b)
	return a.score>b.score
end

function mouseAction(x,y,act)
	----MENU
	
	--NULL INPUT
	--stops it from crashing if theres nothing at click
	if currenttable[nearestMenu(x,y)].action[1] == "null" then
		donothing = donothing
	elseif currenttable[nearestMenu(x,y)].action[1] == "skipintro" then
		sliderinfo[2][1] = 255
	elseif currenttable[nearestMenu(x,y)].action[1] == "game" then
		area = "game"
		currenttable = game
	elseif currenttable[nearestMenu(x,y)].action[1] == "restart" then
		area = "game"
		currenttable = game
		
		loadAssets()
			
		colours = {{0,255,100}}
		--colours[2] = colourreverse(colours[1])
		colours[2] = {0,0,0}
		health = 100
		level = 1
		earlyobjects = {}
		pc = {image = char, pos = {310,300}, vel = {0,0}, phase = 0}
		objects = {}
		arrows = {}
		debug = false
		tick = 0
		status = 0
		speed = 250
		spawned = false
		player = {direction = 0}
		worldsize = 10
		spree = 0
		highest = 0
		datum = 2
		cs = 30
		deads = {deer=0,bears=0,hogs=0}
		damage = 5
		painon = false
		bannertexts = {}
		bannertimer = 0
		banner = " "
		weapon = "bow"
		reload = 101
		firespeed = 100
		emotetimer = 0
		currentemotes = {}
		emote = "null"
		scoreposition = 0
		
		rope.load("obscureworlds/deerhunter1066","settings.lua")
	elseif currenttable[nearestMenu(x,y)].action[1] == "highscores" then
		area = "highscores"
		currenttable = tablehighscores
	elseif currenttable[nearestMenu(x,y)].action[1] == "highscore" then
		typedinput = true
		typed = {}
	elseif currenttable[nearestMenu(x,y)].action[1] == "quakesounds" and deads.deer == 0 and not spawned then
		if settings.quakesounds then
			settings.quakesounds = false
		else
			settings.quakesounds = true
		end
	elseif currenttable[nearestMenu(x,y)].action[1] == "digdugmode" and deads.deer == 0 and not spawned  then
		if settings.digdugmode then
			settings.digdugmode = false
		else
			settings.digdugmode = true
		end
	elseif currenttable[nearestMenu(x,y)].action[1] == "hardmode" and deads.deer == 0 and not spawned  then
		if settings.hardmode then
			settings.hardmode = false
		else
			settings.hardmode = true
		end
	elseif currenttable[nearestMenu(x,y)].action[1] == "fullscreen" and deads.deer == 0 and not spawned  then
		if settings.fullscreen then
			settings.fullscreen = false
			love.graphics.toggleFullscreen()
		else
			settings.fullscreen = true
			love.graphics.toggleFullscreen()
		end
	elseif currenttable[nearestMenu(x,y)].action[1] == "nightmode" and deads.deer == 0 and not spawned  then
		if settings.nightmode then
			settings.nightmode = false
			canvas = nil
		else
			settings.nightmode = true
			canvas = love.graphics.newCanvas(840,640)
		end
	elseif currenttable[nearestMenu(x,y)].action[1] == "resetsave" then
		love.filesystem.remove("settings.lua")
		rope.load("obscureworlds/deerhunter1066","settings.lua")
	elseif currenttable[nearestMenu(x,y)].action[1] == "return" then
		if currenttable == tablehighscores then
			local hsi = "{"
			for i,v in ipairs(highscores) do
				hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
			end
			hsi = hsi.."}"
			love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
			area = "menu"
			currenttable = menu
		elseif currenttable == gameover then
			local temp = " "
			if settings.hardmode then
				temp = "hard"
			end
			if settings.digdugmode then
				temp = temp..",digdug"
			end
			if not settings.digdugmode and not settings.hardmode then
				temp = "normal"
			end
			if settings.nightmode then
				temp = temp..",night"
			end
			if highscorename ~= " " then
				table.insert(highscores,{score=((highest+deads.deer+deads.bears+deads.hogs)/10)*level,highestspree=highest,mode=temp,name=highscorename})
				table.sort(highscores,scoresort)
				table.remove(highscores)
			end
			temp = nil
			highscorename = " "
		
			local hsi = "{"
			for i,v in ipairs(highscores) do
				hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
			end
			hsi = hsi.."}"
			love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
			
			loadAssets()
	
			colours = {{0,255,100}}
			--colours[2] = colourreverse(colours[1])
			colours[2] = {0,0,0}
			health = 100
			level = 1
			earlyobjects = {}
			pc = {image = char, pos = {310,300}, vel = {0,0}, phase = 0}
			objects = {}
			arrows = {}
			debug = false
			tick = 0
			status = 0
			speed = 250
			spawned = false
			player = {direction = 0}
			worldsize = 10
			spree = 0
			highest = 0
			datum = 2
			cs = 30
			deads = {deer=0,bears=0,hogs=0}
			damage = 5
			painon = false
			bannertexts = {}
			bannertimer = 0
			banner = " "
			weapon = "bow"
			reload = 101
			firespeed = 100
			emotetimer = 0
			currentemotes = {}
			emote = "null"
			scoreposition = 0
			
			rope.load("obscureworlds/deerhunter1066","settings.lua")
			
			area = "menu"
			currenttable = menu
		end
	elseif currenttable[nearestMenu(x,y)].action[1] == "quit" then
		local hsi = "{"
		for i,v in ipairs(highscores) do
			hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
		end
		hsi = hsi.."}"
		love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
		love.event.push("quit")
	end
end

-- function that checks if a key is down every game tickyeah
shifton = null
function keydown()
	-- press & release controls
	
	

	-- constant controls
	
	if love.keyboard.isDown("lshift") or love.keyboard.isDown("rshift") then
		shifton = true
	else
		shifton = false
	end
	if not typedinput then
		if settings.hardmode then
			if reload > 100 then
				if control["sup"] then
					keyAction("sup")
				elseif control["sdown"] then
					keyAction("sdown")
				elseif control["sleft"] then
					keyAction("sleft")
				elseif control["sright"] then
					keyAction("sright")
				end
			end
		else
			if control.tap["sup"] then
				keyAction("sup")
			elseif control.tap["sdown"] then
				keyAction("sdown")
			elseif control.tap["sleft"] then
				keyAction("sleft")
			elseif control.tap["sright"] then
				keyAction("sright")
			end
		end
		if control.tap["debug"] then
			if debug then
				debug = false
			else
				debug = true
			end
		end
		if control.tap["return"] then
			colours = {{0,255,100}}
			colours[2] = {0,0,0}
			--colours[2] = colourreverse(colours[1])
		end
		if control.tap["escape"] then
			if area == "menu" then
				local hsi = "{"
				for i,v in ipairs(highscores) do
					hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
				end
				hsi = hsi.."}"
				love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
			elseif area == "game" or area == "highscores" then
				local hsi = "{"
				for i,v in ipairs(highscores) do
					hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
				end
				hsi = hsi.."}"
				love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
				area = "menu"
				currenttable = menu
			elseif area == "gameover" then
				local hsi = "{"
				for i,v in ipairs(highscores) do
					hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
				end
				hsi = hsi.."}"
				love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
				
				loadAssets()
				highscorename = " "
				colours = {{0,255,100}}
				--colours[2] = colourreverse(colours[1])
				colours[2] = {0,0,0}
				health = 100
				level = 1
				earlyobjects = {}
				objects = {{image = char, pos = {310,300}, vel = {0,0}, phase = 0}}
				arrows = {}
				debug = false
				tick = 0
				status = 0
				speed = 250
				spawned = false
				player = {direction = 0}
				worldsize = 10
				spree = 0
				highest = 0
				datum = 2
				cs = 30
				deads = {deer=0,bears=0,hogs=0}
				damage = 5
				painon = false
				bannertexts = {}
				bannertimer = 0
				banner = " "
				weapon = "bow"
				reload = 101
				firespeed = 100
				emotetimer = 0
				currentemotes = {}
				emote = "null"
				scoreposition = 0
				
				rope.load("obscureworlds/deerhunter1066","settings.lua")
				
				area = "menu"
				currenttable = menu
			end
		end
	end
end

function control.controlPressed(c)
	if not typedinput then
		if not settings.hardmode then
			if c == "sup" then
				keyAction("sup")
			end
			if c == "sdown" then
				keyAction("sdown")
			end
			if c == "sleft" then
				keyAction("sleft")
			end
			if c == "sright" then
				keyAction("sright")
			end
		end
		if c == "debug" then
			if debug then
				debug = false
			else
				debug = true
			end
		end
		if c == "return" then
			colours = {{0,255,100}}
			colours[2] = {0,0,0}
			--colours[2] = colourreverse(colours[1])
		end
		if c == "escape" then
			if area == "menu" then
				local hsi = "{"
				for i,v in ipairs(highscores) do
					hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
				end
				hsi = hsi.."}"
				love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
			elseif area == "game" or area == "highscores" then
				local hsi = "{"
				for i,v in ipairs(highscores) do
					hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
				end
				hsi = hsi.."}"
				love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
				area = "menu"
				currenttable = menu
			elseif area == "gameover" then
				local hsi = "{"
				for i,v in ipairs(highscores) do
					hsi = hsi.."{".."score="..v.score..",highestspree="..v.highestspree..",mode=".."'"..v.mode.."'"..",name="..'"'..v.name..'"'.."},"
				end
				hsi = hsi.."}"
				love.filesystem.write("settings.lua","settings="..rope.burn(settings).."\n highscores="..hsi)
				
				loadAssets()
				highscorename = " "
				colours = {{0,255,100}}
				colours[2] = {0,0,0}
				--colours[2] = colourreverse(colours[1])
				health = 100
				level = 1
				earlyobjects = {}
				objects = {{image = char, pos = {310,300}, vel = {0,0}, phase = 0}}
				arrows = {}
				debug = false
				tick = 0
				status = 0
				speed = 250
				spawned = false
				player = {direction = 0}
				worldsize = 10
				spree = 0
				highest = 0
				datum = 2
				cs = 30
				deads = {deer=0,bears=0,hogs=0}
				damage = 5
				painon = false
				bannertexts = {}
				bannertimer = 0
				banner = " "
				weapon = "bow"
				reload = 101
				firespeed = 100
				emotetimer = 0
				currentemotes = {}
				emote = "null"
				scoreposition = 0
				
				rope.load("obscureworlds/deerhunter1066","settings.lua")
				
				area = "menu"
				currenttable = menu
			end
		end
	end
end

function love.keypressed(key)
	if typedinput then
		keyAction("typedinput",key)
	end
end


function keyAction(action,x,y)
	local x = x or false
	local y = y or false
	if action == "typedinput" then
		if x == "backspace" or x == "delete" then
			table.remove(typed)
		else
			if font24:getWidth(table.concat(typed)) <= 225 and typedinputcheck(x) then
				if shifton then
					table.insert(typed,string.upper(x))
				else
					table.insert(typed,x)
				end
			end
		end
		highscorename = table.concat(typed)
	elseif action == "sup" then
		if settings.hardmode then
			reload = 0
		end
		arrowAdd(0,-1)
		love.audio.play(shot2)
	elseif action == "sdown" then
		if settings.hardmode then
			reload = 0
		end
		arrowAdd(0,1)
		love.audio.play(shot2)
	elseif action == "sleft" then
		if settings.hardmode then
			reload = 0
		end
		arrowAdd(-1,0)
		love.audio.play(shot2)
	elseif action == "sright" then
		if settings.hardmode then
			reload = 0
		end
		arrowAdd(1,0)
		love.audio.play(shot2)
	end
end